using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Xml;
using MbUnit.Framework;
using Rss;

namespace MockSubSonic.Tests
{
    [TestFixture]
    public class TechnorariTests
    {
        [Test]
        public void GetTagListFromPostText()
        {
            RssFeed rss = TestHelper.CreateMockRSS(TestHelper.url);
            RssItemCollection items = rss.Channels[0].Items;

            string post = items[0].Description;
            List<string> tags = Technorati.GetTagListFromPost(post);
            Assert.AreEqual(3, tags.Count);
            Assert.AreEqual("MbUnit", tags[0]);
            Assert.AreEqual("TDD", tags[1]);
            Assert.AreEqual("Testing", tags[2]);
        }

        [Test]
        public void Spike_RegExMatchesString()
        {
            Regex r = new Regex("<a href=\"http://technorati.com/tags/(.+)\" rel=\"tag\">(.+)</a>");
            MatchCollection collection =
                r.Matches("<a href=\"http://technorati.com/tags/TestCode\" rel=\"tag\">Test Code</a>");

            Assert.AreEqual(1, collection.Count, "Collection");

            foreach (Match match in collection)
            {
                Assert.IsTrue(match.Success);
                Assert.AreEqual(3, match.Groups.Count, "Captures");
                Assert.AreEqual("TestCode", match.Groups[1].Value, "One");
                Assert.AreEqual("Test Code", match.Groups[2].Value, "Two");
            }
        }

        [Test]
        public void Spike_RegExMatches2Strings()
        {
            Regex r = new Regex("<a href=\"http://technorati.com/tags/(.+?)\" rel=\"tag\">(.+?)</a>", RegexOptions.Multiline);
            MatchCollection collection =
                r.Matches("<a href=\"http://technorati.com/tags/TestCode\" rel=\"tag\">Test Code</a><a href=\"http://technorati.com/tags/TestCode\" rel=\"tag\">Test Code</a>");

            Assert.AreEqual(2, collection.Count, "Collection");

            foreach (Match match in collection)
            {
                Assert.IsTrue(match.Success);
                Assert.AreEqual(3, match.Groups.Count, "Captures");
                Assert.AreEqual("TestCode", match.Groups[1].Value, "One");
                Assert.AreEqual("Test Code", match.Groups[2].Value, "Two");
            }
        }

        [Test]
        public void GetMatchCollectionForHtml()
        {
            string post = "<a href=\"http://technorati.com/tags/TestCode\" rel=\"tag\">Test Code</a>";
            MatchCollection collection = Technorati.GetMatches(post);
            Assert.AreEqual(1, collection.Count, "Collection");
        }

        [Test]
        public void GetListFromMatches()
        {
            string post = "<a href=\"http://technorati.com/tags/TestCode\" rel=\"tag\">Test Code</a>";
            MatchCollection collection = Technorati.GetMatches(post);

            List<string> tags = Technorati.GetTagList(collection);
            Assert.AreEqual(1, tags.Count);
            Assert.AreEqual("TestCode", tags[0]);
        }

        [Test]
        public void GetTechnoratiUrl()
        {

            string technoratiAPI = "7835a030bcb3a4009fcde6f4fece6db5";
            int limit = 10;
            string technoratiTerm = "MbUnit";

            string url = string.Format("http://api.technorati.com/tag?key={0}&tag={1}&limit={2}", technoratiAPI, technoratiTerm, limit);

            string actualUrl = Technorati.GetUrl(technoratiTerm);
            Assert.AreEqual(url, actualUrl);
        }

        [Test]
        public void GetXmlFromRss()
        {
            int limit = 10;
            string technoratiTerm = "MbUnit";
            Technorati.TechnoratiAPI = new TechnoratiStub();
            XmlNodeList xmlNodes = Technorati.GetXmlNodeList(Technorati.GetUrl(technoratiTerm));
            Assert.AreEqual(limit, xmlNodes.Count);
        }

        [Test]
        public void GivenXmlNodeCreateBlog()
        {
            string technoratiTerm = "MbUnit";
            Technorati.TechnoratiAPI = new TechnoratiStub();
            XmlNodeList xmlNodes = Technorati.GetXmlNodeList(Technorati.GetUrl(technoratiTerm));
            Blog blog = Technorati.CreateBlogPost(xmlNodes[0]);
            Assert.AreEqual(blog.Title, xmlNodes[0]["title"].InnerText);
        }

        [Test]
        public void GetBlogPostListTerm()
        {
            string technoratiTerm = "MbUnit";
            Technorati.TechnoratiAPI = new TechnoratiStub();
            XmlNodeList xmlNodes = Technorati.GetXmlNodeList(Technorati.GetUrl(technoratiTerm));
            List<Blog> blogs = Technorati.CreateBlogPostList(xmlNodes);
            Assert.AreEqual(10, blogs.Count);
        }

        [Test]
        public void GetBlogPostListForListOfTerms()
        {
            List<string> terms = new List<string>(3);
            terms.Add("MbUnit");
            terms.Add("TDD");
            terms.Add("Testing");

            Technorati.TechnoratiAPI = new TechnoratiStub();
            List<Blog> blogs = Technorati.CreateBlogPostList(terms);
            Assert.AreEqual(30, blogs.Count);
        }

        [Test]
        public void CreateTechnoratiResponseTest()
        {
            TechnoratiStub stub = new TechnoratiStub();
            XmlNodeList xmlResponse = stub.GetXmlNodeList("");
            Assert.AreEqual(10, xmlResponse.Count);
        }
    }


    [TestFixture]
    public class TestHelper
    {
        internal static string url = "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml";

        [Test]
        public void TestMockCreation()
        {
            RssFeed rss = CreateMockRSS(url);
            Assert.GreaterThan(rss.Channels.Count, 0);
            //Assert.AreEqual(url, rss.Url);  Problem with the understanding on the Framework,  read only getter.  Must use Channels.
            Assert.AreEqual(url, rss.Channels[0].Link.AbsoluteUri);
        }

        internal static RssFeed CreateMockRSS(string url)
        {
            RssFeed rss = new RssFeed();
            rss.Url = url;
            rss.Version = RssVersion.RSS20;

            RssItem ri1a = new RssItem();
            ri1a.Author = "News Reporter A";
            ri1a.Title = "New Item A";
            ri1a.Description = "News Story <b>Important Text</b><a href=\"http://technorati.com/tags/MbUnit\" rel=\"tag\">MbUnit</a>, <a href=\"http://technorati.com/tags/TDD\" rel=\"tag\">TDD</a>, <a href=\"http://technorati.com/tags/Testing\" rel=\"tag\">Testing</a>";
            ri1a.Link = new Uri("http://www.yourserver.com/");
            ri1a.PubDate = DateTime.Now;

            RssItem ri2a = new RssItem();
            ri2a.Author = "News Reporter A";
            ri2a.Title = "New Item A";
            ri2a.Description = "News Story <b>Important Text</b><a href=\"http://technorati.com/tags/MbUnit\" rel=\"tag\">MbUnit</a>, <a href=\"http://technorati.com/tags/TDD\" rel=\"tag\">TDD</a>, <a href=\"http://technorati.com/tags/Testing\" rel=\"tag\">Testing</a>";
            ri2a.Link = new Uri("http://www.yourserver.com/");
            ri2a.PubDate = DateTime.Now;

            RssChannel rc1 = new RssChannel();
            rc1.Items.Add(ri1a);
            rc1.Items.Add(ri2a);
            rc1.Title = "BBC News";
            rc1.Description = "BBC News RSS";
            rc1.Link = new Uri(url);
            rc1.PubDate = DateTime.Now;

            rss.Channels.Add(rc1);

            return rss;
        }
    }

    public class TechnoratiStub : ITechnoratiAccessor
    {
        public XmlNodeList GetXmlNodeList(string technoratiUrl)
        {
            #region Xml
            string cachedXmlResponse = @"<?xml version=""1.0"" encoding=""utf-8""?>" + Environment.NewLine +
                                       @"<!-- generator=""Technorati API version 1.0 /tag"" -->" + Environment.NewLine +
                                       @"<tapi version=""1.0"">" + Environment.NewLine +
                                       @"<document>" + Environment.NewLine +
                                       @"<result>" + Environment.NewLine +
                                       @"  <query>MBunit</query>" + Environment.NewLine +
                                       @"  <postsmatched>10</postsmatched>" +
                                       Environment.NewLine +
                                       @"  <blogsmatched>10</blogsmatched>" +
                                       Environment.NewLine +
                                       @"  <start>1</start>" + Environment.NewLine +
                                       @"  <limit>10</limit>" + Environment.NewLine +
                                       @"  <querytime>00</querytime>" + Environment.NewLine +
                                       @"</result>" + Environment.NewLine +
                                       @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                                                              @"<item>" + Environment.NewLine +
                                       @"  <weblog>" + Environment.NewLine +
                                       @"    <name>[name of blog containing match]</name>" + Environment.NewLine +
                                       @"    <url>[blog URL]</url>" + Environment.NewLine +
                                       @"    <rssurl>[blog RSS URL]</rssurl>" + Environment.NewLine +
                                       @"    <atomurl>[blog Atom URL]</atomurl>" + Environment.NewLine +
                                       @"    <inboundblogs>[inbound blogs]</inboundblogs>" + Environment.NewLine +
                                       @"    <inboundlinks>[inbound links]</inboundlinks>" + Environment.NewLine +
                                       @"    <lastupdate>[date blog last updated]</lastupdate>" + Environment.NewLine +
                                       @"  </weblog>" + Environment.NewLine +
                                       @"  <title>[entry title]</title>" + Environment.NewLine +
                                       @"  <excerpt>[entry blurb with term highlighting]</excerpt>" +
                                       Environment.NewLine +
                                       @"  <created>01/01/2000</created>" + Environment.NewLine +
                                       @"  <postupdate>[date post last updated]</postupdate>" + Environment.NewLine +
                                       @"  <permalink>[blog entry URL]</permalink>" + Environment.NewLine +
                                       @"</item>" + Environment.NewLine +
                                       @"</document>" + Environment.NewLine +
                                       @"</tapi>";
            #endregion

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(cachedXmlResponse);
            return xmlDoc.SelectNodes("/tapi/document/item");
        }
    }
}
